from ._ansi import has_ansi
from ._call import pretty_call
from ._duration import (
    auto_duration_magnitude,
    pretty_duration,
    pretty_duration_unit,
    pretty_durations,
)
from ._func import pretty_func
from ._throughput import pretty_throughput
from ._utils import get_name

__all__ = [
    "auto_duration_magnitude",
    "get_name",
    "has_ansi",
    "pretty_call",
    "pretty_duration",
    "pretty_duration_unit",
    "pretty_durations",
    "pretty_func",
    "pretty_throughput",
]
